@@ -33,7 +33,7 @@ fn main() {
 
     // Read and parse JSON input file
     let file_content =
-        fs::read_to_string("041_rigid_member.json").expect("Failed to read the JSON file");
+        fs::read_to_string("CrossCheckModel.json").expect("Failed to read the JSON file");
     // let file_content = fs::read_to_string("001_cantilever_with_end_load.json").expect("Failed to read the JSON file");
     // let file_content = fs::read_to_string("004_triangular.json").expect("Failed to read the JSON file");
     let mut fers_data: FERS =
