@@ -71,15 +71,17 @@ pub fn compose_support_reaction_vector_equilibrium(
     fers: &FERS,
     result_type: &ResultType,
     displacement_vector_full: &DMatrix<f64>,
+    active_map: Option<&std::collections::HashMap<u32, bool>>,
 ) -> Result<DMatrix<f64>, String> {
     // 1) Deformable member end wrenches → nodes
-    let deformable = deformable_member_ids(fers);
+    let deformable = deformable_member_ids(fers, active_map);
     let ends = member_end_node_map(fers);
 
     let member_results = crate::functions::results::compute_member_results_from_displacement(
         fers,
         result_type,
         displacement_vector_full,
+        active_map,
     );
 
     let mut node_wrench: HashMap<u32, NodeWrench> = HashMap::new();
@@ -93,30 +95,30 @@ pub fn compose_support_reaction_vector_equilibrium(
 
         // Start end
         {
-            let e = node_wrench.entry(n_start).or_default();
-            e.add_force(
-                mr.start_node_forces.fx,
-                mr.start_node_forces.fy,
-                mr.start_node_forces.fz,
+            let entry = node_wrench.entry(n_start).or_default();
+            entry.add_force(
+                -mr.start_node_forces.fx,
+                -mr.start_node_forces.fy,
+                -mr.start_node_forces.fz,
             );
-            e.add_moment(
-                mr.start_node_forces.mx,
-                mr.start_node_forces.my,
-                mr.start_node_forces.mz,
+            entry.add_moment(
+                -mr.start_node_forces.mx,
+                -mr.start_node_forces.my,
+                -mr.start_node_forces.mz,
             );
         }
         // End end
         {
-            let e = node_wrench.entry(n_end).or_default();
-            e.add_force(
-                mr.end_node_forces.fx,
-                mr.end_node_forces.fy,
-                mr.end_node_forces.fz,
+            let entry = node_wrench.entry(n_end).or_default();
+            entry.add_force(
+                -mr.end_node_forces.fx,
+                -mr.end_node_forces.fy,
+                -mr.end_node_forces.fz,
             );
-            e.add_moment(
-                mr.end_node_forces.mx,
-                mr.end_node_forces.my,
-                mr.end_node_forces.mz,
+            entry.add_moment(
+                -mr.end_node_forces.mx,
+                -mr.end_node_forces.my,
+                -mr.end_node_forces.mz,
             );
         }
     }
@@ -279,12 +281,29 @@ fn node_location(fers: &FERS, node_id: u32) -> NodeLocation {
     }
 }
 
-fn deformable_member_ids(fers: &FERS) -> HashSet<u32> {
-    let mut ids = HashSet::new();
+fn deformable_member_ids(
+    fers: &FERS,
+    active_map: Option<&std::collections::HashMap<u32, bool>>,
+) -> std::collections::HashSet<u32> {
+    use crate::models::members::enums::MemberType;
+    let mut ids = std::collections::HashSet::new();
     for ms in &fers.member_sets {
         for m in &ms.members {
-            if !matches!(m.member_type, MemberType::Rigid) {
-                ids.insert(m.id);
+            match m.member_type {
+                MemberType::Rigid => {} // never deformable
+                MemberType::Tension | MemberType::Compression => {
+                    // include only if active (tie/strut logic)
+                    if active_map
+                        .and_then(|mmap| mmap.get(&m.id))
+                        .copied()
+                        .unwrap_or(true)
+                    {
+                        ids.insert(m.id);
+                    }
+                }
+                _ => {
+                    ids.insert(m.id);
+                }
             }
         }
     }
