@@ -1,6 +1,5 @@
 use crate::functions::hinge_and_release_operations::modes_from_single_ends;
 use crate::models::fers::fers::FERS;
-use crate::models::members::enums::MemberType;
 use crate::models::members::memberhinge::AxisMode;
 use crate::models::results::displacement::NodeDisplacement;
 use crate::models::results::forces::NodeForces;
@@ -171,7 +170,10 @@ pub fn compute_member_results_from_displacement(
     fers: &FERS,
     result_type: &ResultType,
     global_displacement_vector: &DMatrix<f64>,
+    active_map: Option<&std::collections::HashMap<u32, bool>>,
 ) -> BTreeMap<u32, MemberResult> {
+    use crate::models::members::enums::MemberType;
+
     let (material_map, section_map, hinge_map, _support_map) = fers.build_lookup_maps();
     let mut results_by_member_identifier: BTreeMap<u32, MemberResult> = BTreeMap::new();
 
@@ -191,23 +193,97 @@ pub fn compute_member_results_from_displacement(
                     global_displacement_vector[(end_node_dof_index + dof, 0)];
             }
 
-            // Base local K (BEFORE releases)
+            if matches!(
+                member.member_type,
+                MemberType::Truss | MemberType::Tension | MemberType::Compression
+            ) {
+                // inactive ties/struts → zero forces
+                let is_active = active_map
+                    .and_then(|m| m.get(&member.id).copied())
+                    .unwrap_or(true);
+
+                if matches!(
+                    member.member_type,
+                    MemberType::Tension | MemberType::Compression
+                ) && !is_active
+                {
+                    let zeros = NodeForces {
+                        fx: 0.0,
+                        fy: 0.0,
+                        fz: 0.0,
+                        mx: 0.0,
+                        my: 0.0,
+                        mz: 0.0,
+                    };
+                    results_by_member_identifier.insert(
+                        member.id,
+                        MemberResult {
+                            start_node_forces: zeros,
+                            end_node_forces: zeros,
+                            maximums: zeros,
+                            minimums: zeros,
+                        },
+                    );
+                    continue;
+                }
+
+                // active: compute axial force (tension +)
+                let n = member.calculate_axial_force_3d(
+                    global_displacement_vector,
+                    &material_map,
+                    &section_map,
+                );
+
+                // unit direction (global) from start → end
+                let (dx, dy, dz) = {
+                    let length = member.calculate_length() as f64;
+                    let ux = (member.end_node.X - member.start_node.X) / length;
+                    let uy = (member.end_node.Y - member.start_node.Y) / length;
+                    let uz = (member.end_node.Z - member.start_node.Z) / length;
+                    (ux, uy, uz)
+                };
+
+                let start = NodeForces {
+                    fx: n * dx,
+                    fy: n * dy,
+                    fz: n * dz,
+                    mx: 0.0,
+                    my: 0.0,
+                    mz: 0.0,
+                };
+                let end = NodeForces {
+                    fx: -n * dx,
+                    fy: -n * dy,
+                    fz: -n * dz,
+                    mx: 0.0,
+                    my: 0.0,
+                    mz: 0.0,
+                };
+                let (maximums, minimums) = compute_component_extrema(&start, &end);
+
+                results_by_member_identifier.insert(
+                    member.id,
+                    MemberResult {
+                        start_node_forces: start,
+                        end_node_forces: end,
+                        maximums,
+                        minimums,
+                    },
+                );
+                continue;
+            }
+
+            // === Normal (beam) path with semi-rigid ends ===
             let k_local_base =
                 match member.calculate_stiffness_matrix_3d(&material_map, &section_map) {
                     Some(k) => k,
-                    None => {
-                        // For non-beam elements (e.g., truss), you may want a separate path.
-                        // For now, skip cleanly.
-                        continue;
-                    }
+                    None => continue,
                 };
 
             let t = member.calculate_transformation_matrix_3d();
-            let u_local_node = &t * &u_member_global; // node-side DoFs (what the solver sees)
+            let u_local_node = &t * &u_member_global;
 
-            // Equivalent nodal loads in LOCAL axes (only distributed loads contribute here)
             let mut f_eq_local = DVector::<f64>::zeros(12);
-
             match result_type {
                 ResultType::Loadcase(load_case_id) => {
                     if let Some(load_case) =
@@ -303,7 +379,6 @@ pub fn compute_member_results_from_displacement(
                                 fers.load_cases.iter().find(|x| x.id == *case_id)
                             {
                                 let mut v_case = DVector::<f64>::zeros(12);
-
                                 for distributed_load in &load_case.distributed_loads {
                                     if distributed_load.member != member.id
                                         || distributed_load.load_case != *case_id
@@ -389,14 +464,14 @@ pub fn compute_member_results_from_displacement(
                 }
             }
 
-            // Hinge modes (per end, local axes)
+            // hinge modes (per end, local axes)
             let a_h = member
                 .start_hinge
                 .and_then(|id| hinge_map.get(&id).copied());
             let b_h = member.end_hinge.and_then(|id| hinge_map.get(&id).copied());
             let (a_trans, a_rot, b_trans, b_rot) = modes_from_single_ends(a_h, b_h);
 
-            // --- Semi-rigid recovery ---
+            // recover member forces in LOCAL with semi-rigid ends
             let q_local_member = recover_member_end_forces_local(
                 &k_local_base,
                 &u_local_node,
@@ -429,7 +504,6 @@ pub fn compute_member_results_from_displacement(
 
             let (maximums, minimums) =
                 compute_component_extrema(&start_node_forces, &end_node_forces);
-
             results_by_member_identifier.insert(
                 member.id,
                 MemberResult {
