@@ -383,17 +383,6 @@ impl FERS {
             }
         }
 
-        // === Optional: tiny sanity print for your exact case (remove when satisfied) ===
-        if cfg!(debug_assertions) {
-            let uy_b = FERS::dof_index(2, 1); // node 2, UY
-            let rz_a = FERS::dof_index(1, 5); // node 1, RZ
-            if let Some(col_rz_a) = full_to_red.get(&rz_a) {
-                let coeff = s[(uy_b, *col_rz_a)];
-                log::debug!("S[node2 UY, node1 RZ] = {}", coeff);
-                // expect +2.5 for your geometry
-            }
-        }
-
         Ok(RigidElimination { s, full_to_red })
     }
 
@@ -762,69 +751,226 @@ impl FERS {
         rhs[(dof_index, 0)] = prescribed;
     }
 
+    fn constrain_linear_constraint_penalty(
+        &self,
+        k_red: &mut nalgebra::DMatrix<f64>,
+        rhs_red: &mut nalgebra::DMatrix<f64>,
+        a_column: &nalgebra::DMatrix<f64>, // shape (n_red, 1)
+        prescribed: f64,
+        penalty_factor: f64,
+    ) {
+        let n_red_rows: usize = k_red.nrows();
+        let n_red_cols: usize = k_red.ncols();
+        debug_assert_eq!(n_red_rows, n_red_cols, "Reduced stiffness must be square.");
+        debug_assert_eq!(
+            a_column.nrows(),
+            n_red_rows,
+            "Constraint vector length must match reduced system size."
+        );
+        debug_assert_eq!(a_column.ncols(), 1, "Constraint vector must be a column.");
+
+        // Scale penalty from the matrix magnitude to mitigate conditioning issues.
+        let mut max_diag: f64 = 0.0;
+        for i in 0..n_red_rows {
+            let value: f64 = k_red[(i, i)].abs();
+            if value > max_diag {
+                max_diag = value;
+            }
+        }
+        if max_diag <= 0.0 {
+            max_diag = 1.0;
+        }
+        let alpha: f64 = penalty_factor * max_diag;
+
+        // K += alpha * a * a^T
+        for i in 0..n_red_rows {
+            let ai: f64 = a_column[(i, 0)];
+            if ai == 0.0 {
+                continue;
+            }
+            for j in 0..n_red_rows {
+                let aj: f64 = a_column[(j, 0)];
+                if aj == 0.0 {
+                    continue;
+                }
+                k_red[(i, j)] += alpha * ai * aj;
+            }
+        }
+
+        // rhs += alpha * a * prescribed
+        if prescribed != 0.0 {
+            for i in 0..n_red_rows {
+                rhs_red[(i, 0)] += alpha * a_column[(i, 0)] * prescribed;
+            }
+        }
+    }
+
+    /// Try to detect whether a constraint vector is essentially "one-hot" on a single reduced DOF.
+    /// Returns Some(pivot_index) if exactly or approximately one-hot, otherwise None.
+    fn detect_one_hot_constraint(
+        &self,
+        a_column: &nalgebra::DMatrix<f64>,
+        tolerance_ratio: f64,
+    ) -> Option<usize> {
+        debug_assert_eq!(a_column.ncols(), 1, "Constraint vector must be a column.");
+        let n: usize = a_column.nrows();
+
+        // Find the entry with the largest absolute value
+        let mut max_val: f64 = 0.0;
+        let mut max_idx: usize = 0;
+        for i in 0..n {
+            let v: f64 = a_column[(i, 0)].abs();
+            if v > max_val {
+                max_val = v;
+                max_idx = i;
+            }
+        }
+        if max_val == 0.0 {
+            return None;
+        }
+
+        // Sum of squares of all other coefficients
+        let mut sum_sq_other: f64 = 0.0;
+        for i in 0..n {
+            if i == max_idx {
+                continue;
+            }
+            let v: f64 = a_column[(i, 0)];
+            sum_sq_other += v * v;
+        }
+
+        // If the energy of others is small relative to the pivot, treat it as one-hot
+        if sum_sq_other <= (tolerance_ratio * tolerance_ratio) * (max_val * max_val) {
+            Some(max_idx)
+        } else {
+            None
+        }
+    }
+
     fn apply_boundary_conditions_reduced(
         &self,
         elim: &RigidElimination,
-        k_red: &mut DMatrix<f64>,
-        rhs_red: &mut DMatrix<f64>,
+        k_red: &mut nalgebra::DMatrix<f64>,
+        rhs_red: &mut nalgebra::DMatrix<f64>,
     ) -> Result<(), String> {
+        use crate::models::supports::supportconditiontype::SupportConditionType;
+        use std::collections::HashMap;
+
         let support_map: HashMap<u32, &NodalSupport> =
             self.nodal_supports.iter().map(|s| (s.id, s)).collect();
 
+        // Tunables: penalty factor and one-hot tolerance
+        const BC_PENALTY_FACTOR: f64 = 1.0e8; // increase if constraints look too soft; decrease if conditioning suffers
+        const ONE_HOT_TOL_RATIO: f64 = 1.0e-6; // if all other coefficients are <= 1e-6 of the pivot, treat as one-hot
+
+        let n_red: usize = k_red.nrows();
+        debug_assert_eq!(n_red, k_red.ncols(), "Reduced stiffness must be square.");
+        debug_assert_eq!(
+            rhs_red.nrows(),
+            n_red,
+            "RHS size must match reduced system size."
+        );
+        debug_assert_eq!(rhs_red.ncols(), 1, "RHS must be a column vector.");
+
         for ms in &self.member_sets {
             for m in &ms.members {
                 for node in [&m.start_node, &m.end_node] {
-                    let Some(sid) = node.nodal_support else {
+                    let Some(support_id) = node.nodal_support else {
                         continue;
                     };
-                    let Some(s) = support_map.get(&sid) else {
+                    let Some(support) = support_map.get(&support_id) else {
                         continue;
                     };
 
-                    let base_full = (node.id as usize - 1) * 6;
+                    let base_full: usize = (node.id as usize - 1) * 6;
 
-                    for (axis, dof) in TRANSLATION_AXES {
-                        let cond = s
-                            .displacement_conditions
-                            .get(axis)
-                            .or_else(|| s.displacement_conditions.get(&axis.to_ascii_lowercase()));
-                        let is_fixed = cond
+                    // Handle translational constraints
+                    for (axis_label, local_dof) in TRANSLATION_AXES {
+                        let cond_opt =
+                            support.displacement_conditions.get(axis_label).or_else(|| {
+                                support
+                                    .displacement_conditions
+                                    .get(&axis_label.to_ascii_lowercase())
+                            });
+                        let is_fixed: bool = cond_opt
                             .map(|c| matches!(c.condition_type, SupportConditionType::Fixed))
                             .unwrap_or(true);
-                        if is_fixed {
-                            let fi = base_full + dof;
-                            let Some(ri) = elim.full_to_red.get(&fi).copied() else {
-                                return Err(format!(
-                                    "Support at slave node {} (DOF {}). Not supported yet.",
-                                    node.id, axis
-                                ));
-                            };
+                        if !is_fixed {
+                            continue;
+                        }
+
+                        let fi: usize = base_full + local_dof;
+                        if let Some(ri) = elim.full_to_red.get(&fi).copied() {
+                            // Retained DOF: constrain exactly
                             self.constrain_single_dof(k_red, rhs_red, ri, 0.0);
+                        } else {
+                            // Slave DOF: enforce (S_row * u_red) = 0
+                            let mut a_column = nalgebra::DMatrix::<f64>::zeros(n_red, 1);
+                            for j in 0..n_red {
+                                a_column[(j, 0)] = elim.s[(fi, j)];
+                            }
+
+                            // If the constraint is essentially one-hot, constrain that single DOF exactly.
+                            if let Some(pivot_j) =
+                                self.detect_one_hot_constraint(&a_column, ONE_HOT_TOL_RATIO)
+                            {
+                                self.constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
+                            } else {
+                                self.constrain_linear_constraint_penalty(
+                                    k_red,
+                                    rhs_red,
+                                    &a_column,
+                                    0.0, // prescribed displacement is zero
+                                    BC_PENALTY_FACTOR,
+                                );
+                            }
                         }
                     }
 
-                    for (axis, dof) in ROTATION_AXES {
-                        let cond = s
-                            .rotation_conditions
-                            .get(axis)
-                            .or_else(|| s.rotation_conditions.get(&axis.to_ascii_lowercase()));
-                        let is_fixed = cond
+                    // Handle rotational constraints
+                    for (axis_label, local_dof) in ROTATION_AXES {
+                        let cond_opt = support.rotation_conditions.get(axis_label).or_else(|| {
+                            support
+                                .rotation_conditions
+                                .get(&axis_label.to_ascii_lowercase())
+                        });
+                        let is_fixed: bool = cond_opt
                             .map(|c| matches!(c.condition_type, SupportConditionType::Fixed))
                             .unwrap_or(true);
-                        if is_fixed {
-                            let fi = base_full + dof;
-                            let Some(ri) = elim.full_to_red.get(&fi).copied() else {
-                                return Err(format!(
-                                    "Support at slave node {} (Rot {}). Not supported yet.",
-                                    node.id, axis
-                                ));
-                            };
+                        if !is_fixed {
+                            continue;
+                        }
+
+                        let fi: usize = base_full + local_dof;
+                        if let Some(ri) = elim.full_to_red.get(&fi).copied() {
+                            // Retained rotational DOF: constrain exactly
                             self.constrain_single_dof(k_red, rhs_red, ri, 0.0);
+                        } else {
+                            // Slave rotational DOF: enforce (S_row * u_red) = 0
+                            let mut a_column = nalgebra::DMatrix::<f64>::zeros(n_red, 1);
+                            for j in 0..n_red {
+                                a_column[(j, 0)] = elim.s[(fi, j)];
+                            }
+
+                            if let Some(pivot_j) =
+                                self.detect_one_hot_constraint(&a_column, ONE_HOT_TOL_RATIO)
+                            {
+                                self.constrain_single_dof(k_red, rhs_red, pivot_j, 0.0);
+                            } else {
+                                self.constrain_linear_constraint_penalty(
+                                    k_red,
+                                    rhs_red,
+                                    &a_column,
+                                    0.0,
+                                    BC_PENALTY_FACTOR,
+                                );
+                            }
                         }
                     }
                 }
             }
         }
+
         Ok(())
     }
 
@@ -855,6 +1001,8 @@ impl FERS {
         map
     }
 
+    // Do not abbreviate code
+
     fn solve_first_order_common(
         &mut self,
         load_vector_full: nalgebra::DMatrix<f64>,
@@ -870,24 +1018,29 @@ impl FERS {
 
         let assembly_context: AssemblyContext<'_> = AssemblyContext::new(self);
 
+        // Build rigid elimination (S) once; it depends only on topology/hinges
         let elim = self.build_rigid_elimination_partial_using_hinges()?;
 
         let mut converged = false;
         for _iter in 0..max_it {
+            // Linear operator (no geometric stiffness) for first-order analysis
             let k_full = self.build_operator_with_supports(&active_map, None)?;
 
+            // Reduce system: K_r = Sᵀ K S,  f_r = Sᵀ f
             let (mut k_red, mut f_red) = Self::reduce_system(&k_full, &load_vector_full, &elim);
 
-            // apply BCs on the REDUCED system
+            // Apply boundary conditions in REDUCED space
             self.apply_boundary_conditions_reduced(&elim, &mut k_red, &mut f_red)?;
 
-            // solve reduced system and expand to full DOFs
+            // Solve reduced system
             let u_red = k_red.lu().solve(&f_red).ok_or_else(|| {
                 "Reduced stiffness matrix is singular or near-singular".to_string()
             })?;
+
+            // Expand to FULL space
             let u_full_new = Self::expand_solution(&elim, &u_red);
 
-            // active-set update (Tension/Compression) uses FULL displacement
+            // Active-set update (Tension/Compression) uses FULL displacement
             let delta = &u_full_new - &u_full;
             u_full = u_full_new;
 
@@ -912,18 +1065,69 @@ impl FERS {
             ));
         }
 
-        // reactions from the linear operator in FULL space
-        let composed_support_reaction_vector_full =
-            compose_support_reaction_vector_equilibrium(self, &result_type, &u_full)?;
+        // ---------------------------
+        // Build reactions including MPCs
+        // ---------------------------
+        // Recompute the FINAL linear operator with the final active_map,
+        // then form the reduced residual r_r = K_r u_r - f_r with the UNMODIFIED K_r, f_r
+        // Map to FULL space: r_full = S r_red
+        // Keep only true support reactions
+        let r_support = compose_support_reaction_vector_equilibrium(
+            self,
+            &result_type,      // <- pass the ResultType you're solving
+            &u_full,           // <- final full displacement vector
+            Some(&active_map), // <- so ties/struts respect active set
+        )?;
 
+        let mut sum_rx = 0.0;
+        let mut sum_ry = 0.0;
+        let mut sum_rz = 0.0;
+        for (i, val) in r_support.iter().enumerate() {
+            match i % 6 {
+                0 => sum_rx += val,
+                1 => sum_ry += val,
+                2 => sum_rz += val,
+                _ => {}
+            }
+        }
+
+        // Sum of EXTERNAL loads per axis (global vector you assembled)
+        let mut sum_fx = 0.0;
+        let mut sum_fy = 0.0;
+        let mut sum_fz = 0.0;
+        for (i, val) in load_vector_full.iter().enumerate() {
+            match i % 6 {
+                0 => sum_fx += val,
+                1 => sum_fy += val,
+                2 => sum_fz += val,
+                _ => {}
+            }
+        }
+
+        log::debug!(
+            "Equil check: sum reactions [Fx,Fy,Fz] = [{:.6}, {:.6}, {:.6}]",
+            sum_rx,
+            sum_ry,
+            sum_rz
+        );
+        log::debug!(
+            "Equil check: sum external [Fx,Fy,Fz]  = [{:.6}, {:.6}, {:.6}]",
+            sum_fx,
+            sum_fy,
+            sum_fz
+        );
+
+        // Store masked reactions
         let results = self
             .build_and_store_results(
                 name.clone(),
                 result_type.clone(),
                 &u_full,
-                &composed_support_reaction_vector_full,
+                &r_support,
+                Some(&active_map),
             )?
             .clone();
+
         Ok(results)
     }
 
@@ -943,36 +1147,40 @@ impl FERS {
 
         let assembly_context: AssemblyContext<'_> = AssemblyContext::new(self);
 
+        // Build rigid elimination (S) once; it depends only on topology/hinges
         let elim = self.build_rigid_elimination_partial_using_hinges()?;
 
         let mut converged = false;
         for _iter in 0..max_iterations {
+            // Tangent operator (includes geometric stiffness) for Newton–Raphson
             let k_tangent_full = self.build_operator_with_supports(&active_map, Some(&u_full))?;
 
-            let (k_red, f_red) = Self::reduce_system(&k_tangent_full, &load_vector_full, &elim);
+            // Reduce: K_tr = Sᵀ K_tangent S, f_r = Sᵀ f
+            let (k_red_tangent, f_red) =
+                Self::reduce_system(&k_tangent_full, &load_vector_full, &elim);
 
             // Current reduced displacement u_r = Sᵀ u
             let mut u_red = elim.s.transpose() * &u_full;
 
-            // Residual in reduced space: r_r = K_r u_r − f_r
-            let mut r_red = &k_red * &u_red - &f_red;
+            // Residual in reduced space (before BC application): r_r = K_tr u_r − f_r
+            let mut r_red = &k_red_tangent * &u_red - &f_red;
 
-            // Apply BCs in REDUCED space, then solve for Δu_r from K_r Δu_r = −r_r
-            let mut k_treated = k_red.clone();
+            // Apply BCs in REDUCED space, then solve for Δu_r from K_tr,BC Δu_r = −r_r,BC
+            let mut k_treated = k_red_tangent.clone();
             self.apply_boundary_conditions_reduced(&elim, &mut k_treated, &mut r_red)?;
             let delta_red = k_treated
                 .lu()
                 .solve(&(-&r_red))
                 .ok_or_else(|| "Tangent stiffness singular.".to_string())?;
 
-            // Update and expand back to full space
+            // Update reduced, expand to FULL
             u_red += &delta_red;
             let u_full_new = Self::expand_solution(&elim, &u_red);
 
+            // Check convergence and update active set
             let delta_full = &u_full_new - &u_full;
             u_full = u_full_new;
 
-            // Active-set update (Tension/Compression) using FULL displacement
             let changed = self.update_active_set(
                 &u_full,
                 &mut active_map,
@@ -994,16 +1202,26 @@ impl FERS {
             ));
         }
 
-        // Reactions from the linear operator (no geometric part), FULL space
-        let composed_support_reaction_vector_full =
-            compose_support_reaction_vector_equilibrium(self, &result_type, &u_full)?;
+        // ---------------------------
+        // Build reactions including MPCs
+        // ---------------------------
+        // Use the LINEAR operator (no geometric stiffness) to compute final reactions,
+        // matching classic practice and your previous intent/comments.
+        // Map to FULL space reactions
+        let r_support = compose_support_reaction_vector_equilibrium(
+            self,
+            &result_type,      // <- pass the ResultType you're solving
+            &u_full,           // <- final full displacement vector
+            Some(&active_map), // <- so ties/struts respect active set
+        )?;
 
         let results = self
             .build_and_store_results(
                 name.clone(),
                 result_type.clone(),
                 &u_full,
-                &composed_support_reaction_vector_full,
+                &r_support,
+                Some(&active_map),
             )?
             .clone();
         Ok(results)
@@ -1077,77 +1295,30 @@ impl FERS {
         )
     }
 
-    /// Compose support reactions by **equilibrium per node**, not from residuals.
-    /// - Deformable members → sum end forces AND end moments into their end nodes
-    /// - Add applied **nodal** loads and **nodal** moments (cases or combinations)
-    /// - Roll up wrenches along **rigid links** (F_a += F_b ; M_a += M_b + r × F_b)
-    /// - For each support DOF:
-    ///     Fixed           → reaction = −(net node wrench component)
-    ///     Spring          → reaction = k * dof_value   (existing spring logic)
-    ///     Free            → 0
-    ///     PositiveOnly    → clamp ≥ 0
-    ///     NegativeOnly    → clamp ≤ 0
     pub fn build_and_store_results(
         &mut self,
         name: String,
         result_type: ResultType,
         displacement_vector: &DMatrix<f64>,
         global_reaction_vector: &DMatrix<f64>,
+        active_map: Option<&std::collections::HashMap<u32, bool>>,
     ) -> Result<&Results, String> {
-        // 1) Element/member results + basic node maps
-        let member_results =
-            compute_member_results_from_displacement(self, &result_type, displacement_vector);
+        // 1) Element/member results
+        let member_results = compute_member_results_from_displacement(
+            self,
+            &result_type,
+            displacement_vector,
+            active_map,
+        );
+
+        // 2) Node displacements
         let displacement_nodes = extract_displacements(self, displacement_vector);
-        let reaction_nodes = extract_reaction_nodes(self, global_reaction_vector);
 
-        // ---- member_id -> (start_node_id, end_node_id)
-        let mut member_ends: HashMap<u32, (u32, u32)> = HashMap::new();
-        for ms in &self.member_sets {
-            for m in &ms.members {
-                member_ends.insert(m.id, (m.start_node.id, m.end_node.id));
-            }
-        }
+        // 3) Reactions: already computed via compose_support_reaction_vector_equilibrium(...)
+        let reaction_nodes: BTreeMap<u32, crate::models::results::reaction::ReactionNodeResult> =
+            extract_reaction_nodes(self, global_reaction_vector);
 
-        // 2) Net nodal forces coming from **deformable** members only
-
-        let mut node_force: HashMap<u32, (f64, f64, f64)> = HashMap::new();
-        for (mid, mr) in &member_results {
-            let Some(&(n_start, n_end)) = member_ends.get(mid) else {
-                continue;
-            };
-
-            let e = node_force.entry(n_start).or_insert((0.0, 0.0, 0.0));
-            e.0 += mr.start_node_forces.fx;
-            e.1 += mr.start_node_forces.fy;
-            e.2 += mr.start_node_forces.fz;
-
-            let e = node_force.entry(n_end).or_insert((0.0, 0.0, 0.0));
-            e.0 += mr.end_node_forces.fx;
-            e.1 += mr.end_node_forces.fy;
-            e.2 += mr.end_node_forces.fz;
-        }
-
-        // 3) Build rigid-link parent map: slave -> (master, r = x_b - x_a)
-        let mut parent: HashMap<u32, (u32, (f64, f64, f64))> = HashMap::new();
-        let mut rigid_edges: Vec<(u32, u32, (f64, f64, f64))> = Vec::new(); // (a, b, r_ab)
-        for ms in &self.member_sets {
-            for m in &ms.members {
-                if !matches!(m.member_type, MemberType::Rigid) {
-                    continue;
-                }
-                let a = m.start_node.id;
-                let b = m.end_node.id;
-                let r = (
-                    m.end_node.X - m.start_node.X,
-                    m.end_node.Y - m.start_node.Y,
-                    m.end_node.Z - m.start_node.Z,
-                );
-                parent.insert(b, (a, r));
-                rigid_edges.push((a, b, r));
-            }
-        }
-
-        // 5) Pack results & store
+        // 4) Pack results & store
         let total_members: usize = self.member_sets.iter().map(|set| set.members.len()).sum();
         let total_supports: usize = self.nodal_supports.len();
 
